library("preprocessCore")

wd = "/pbld/mcg/lillianpetersen/" # working directory

MCG = FALSE
All = FALSE
BALL = TRUE
TALL = FALSE
AML = FALSE

if(MCG){
	nRows = 271911
	nSamples = 20
	sampleNames = c('MCG001','MCG002','MCG003','MCG004','MCG005','MCG006','MCG009','MCG010','MCG011','MCG012','MCG013','MCG015','MCG016','MCG017','MCG019','MCG020','MCG023','MCG024','MCG025','MCG026')
	peak_file = paste(wd,"peak_allignment/merged/",sep="")
}
if(All){
	nRows = #update
	nSamples = 23
	Type = 'All'
	sampleNames = c('BALL_control','TALL_control','AML_control','MCG001','MCG002','MCG003','MCG004','MCG005','MCG006','MCG009','MCG010','MCG011','MCG012','MCG013','MCG015','MCG016','MCG017','MCG019','MCG020','MCG023','MCG024','MCG025','MCG026')
	peak_file = paste(wd,"peak_allignment/with_control/All/",sep="")
}
if(BALL){
	nRows = 251294
	nSamples = 25
	Type = 'B_ALL'
	sampleNames = c('MCG001','MCG002','MCG003','MCG005','MCG006','MCG009','MCG010','MCG011','MCG012','MCG013','MCG016','MCG017','MCG019','MCG020','MCG023','MCG024','MCG027','MCG028','MCG029','MCG034','MCG035','MCG036','MCG037','MCG038','MCG039')
	peak_file = paste(wd,"peak_allignment/B_cell/",sep="")
}
if(TALL){
	nRows = 184567
	nSamples = 4
	Type = 'T_ALL'
	sampleNames = c('TALL_control','MCG015','MCG025','MCG026')
	peak_file = paste(wd,"peak_allignment/with_control/T_ALL/",sep="")
}
if(AML){
	nRows = 265698
	nSamples = 2
	Type = 'AML'
	sampleNames = c('AML_control','MCG004')
	peak_file = paste(wd,"peak_allignment/with_control/AML/",sep="")
}

rpkm = matrix(nrow=nRows,ncol=nSamples) # initialize array of peaks. size = # peaks, # samples
i = 0
for (sample in sampleNames){
	i=i+1
	rpkmArray = read.csv(file=paste(peak_file,sample,"_rpkm_merged_counts.txt",sep=""), header=FALSE, sep="\t")
	rpkm[,i] = rpkmArray[,4] # read unnormalized peaks into array
}

# Plot unnormalized peaks
plot.new()
pdf( paste(wd,"figures/peaks_before_normalization_",Type,".pdf",sep="") )
#cols = c('#32CD32','#FF0000','#FFA500','#00FA9A','#00FFFF','#008080','#FF00FF','#800080','#F4A460','#FFFF00','#000080','#DDA0DD','#FFE4E1','#800000','#3CB371','#1E90FF')
cols = c('#e6194b', '#3cb44b', '#ffe119', '#4363d8', '#f58231', '#911eb4', '#46f0f0', '#f032e6', '#bcf60c', '#fabebe', '#008080', '#e6beff', '#9a6324', '#fffac8', '#800000', '#aaffc3', '#808000', '#ffd8b1', '#000075', '#808080', '#000000', '#7B68EE', '#C67171')
plot(density(rpkm[,1],bw=0.2),
		col=cols[1],lwd=3,
		ylim=c(0,.45),xlim=c(-5,20),
		main=paste(Type,': Peaks Before Normalization'),
		xlab = 'rpkm' )
grid(nx = NULL, ny = NULL, col = "lightgray", lty = "dotted", lwd = par("lwd"), equilogs = TRUE)
for(k in 2:nSamples){lines(density(rpkm[,k],bw=0.2),lwd=3,col=cols[k])}
dev.off()

norm_rpkm= normalize.quantiles(rpkm) # quantile normalize samples
plot.new()
pdf( paste(wd,"figures/peaks_after_normalization_",Type,".pdf",sep="") )
# Plot normalized peaks
plot(density(norm_rpkm[,1],bw=0.2),
		col=cols[1],lwd=3,
		ylim=c(0,.45),xlim=c(-5,20),
		main=paste(Type,': Peaks After Normalization'),
		xlab = 'Quantile Normalized rpkm' )
grid(nx = NULL, ny = NULL, col = "lightgray", lty = "dotted", lwd = par("lwd"), equilogs = TRUE)
for(k in 2:nSamples){lines(density(norm_rpkm[,k],bw=0.2),lwd=3,col=cols[k])}
dev.off()

# Write normalized peaks into file
i = 0
for (sample in sampleNames){
	i=i+1
	# write to a file
	write.table(data.frame(rpkmArray$V1,rpkmArray$V2,rpkmArray$V3,norm_rpkm[,i]), file = paste(peak_file,sample,"_merged_counts.rpkm.qn.txt",sep=""), sep = '\t',col.names = FALSE, row.names = FALSE, quote=FALSE)

}

